#!/bin/bash

# --------------------------------------------------- User Manual ---------------------------------------------------
#
# This script creates a slurm.annotate file for variant annotation.
# The "sed" lines below set parameters for annotation. Please review each line and change it for your job.
#
# Basic parameters:
# * The MY_EMAIL and MY_ACCOUNT lines are used for the job scheduler in your institution.
#   Those lines are not useful when you intend to run the script directly and can be safely removed.
#   Otherwise please replace the your.email@utah.edu your_account your_partition with correct values.
# * "module load fenglab" is how we load the PATH for VICTOR within the University of Utah. 
#   If you have a different setup, change it throughout this file based on your installation.
# * The last two lines set the input file to multi_chr.vcf.gz and the output prefix to VICTOR.
#   OUT can contain a full path, e.g.: /path/to/results/output, the output archive is /path/to/results/output.zip.
#   Beware, there should not be any files with the prefix of OUT in your destination path, otherwise they will all
#   be removed after running the script.
#
# Other parameters to consider:
# * If your VCF file is small, add this line to speed up: sed "s/USE_TABIX:-/USE_TABIX:-yes/" |\
# * To annoate ACMG genes only: sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --prefer=panel_incidental" ;' |\
# * To annoate only a few genes: sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --genes=BRCA1,BRCA2" ;' |\
# * "gbags.cancer" is suitable for jobs focusing on cancer. Remove that line if you study other diseases.
# * If you don't want BayesDel to include MaxAF: sed 's/export DEL=             /export DEL="--add-af=no"/' |\
# * To sorted by #CHROM,POS like a VCF: sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --sort-bp" ;' |\
# * To remove non-functional variants: sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --rm-nonfunc" ;' |\
# * To put all annotations in INFO: sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --add-info" ;' |\
# * To skip running PROVEAN for in-frame InDel: sed 's;^export PVN=   ;export PVN=no ;' |\
# * To output column 1-8 only: sed 's;^export KEEP_DATA=[^ ]\+ ;export KEEP_DATA=no ;' |\
# * To keep the original coordinate: sed 's;^export ORIG_COOR=[^ ]\+ ;export ORIG_COOR=yes ;' |\
#
# Genomic build:
#   slurm.annotate looks for indication of genomic build in the VCF and automatically decide which database to use. 
#   If the VCF does not contain genomic build, the program will look at the full path of the working directory.
#   If all these have failed, the program will complain and stop running.
#
# After running slurm.annotate:
#   The output archive is VICTOR.zip. Here, VICTOR is the output prefix specified by the OUT parameter.
#   Unzip VICTOR.zip will create a folder named VICTOR. The annotated file is VICTOR/VICTOR.ann.del.gz.
#   This file is sorted and indexed by gene symbols. You can extract a gene by "tar VICTOR.ann.del.gz <GeneSymbol>".
#   Other files in the folder are log files, including slurm.annotate.run_1.stderr. See inside this file for errors.
#
# Note:
# * Output may have more lines than input because multi-ALT variants are split into multiple lines.
# * Some input variants are not found in the output because the pipeline changes the coordinate of some variants. 
#   These variants are InDels located in a repeated sequence. Most bioinformatics pipelines align these variants 
#   to the left-most position so that they can be uniquely identified, without considering the functional consequence 
#   of the variants. As a package focusing on the secondary and tertiary analyses, VICTOR shifts the coordinate for a 
#   better representation of the functional consequence of the variants, whenever it is needed. If you want to match 
#   the output to the input, then you can retore the original coordinate by setting ORIG_COOR=yes (see above). 
#   You can also search for the OriginalIndex field in the INFO column just to view the original coordinate.
# ------------------------------------------------- End User Manual -------------------------------------------------

module load fenglab

cat /path/to/VICTOR/slurm.annotate |\
sed "s/MY_EMAIL/your.name@your.university.edu/" |\
sed "s/#SBATCH --account=MY_ACCOUNT/#SBATCH --account=your_account\n#SBATCH --partition=your_partition/" |\
sed "s/^export GBA=             /export GBA=gbags.cancer /" |\
sed 's;^export VAG="\([^"]\+\)" ;export VAG="\1 --sort-bp" ;' |\
sed "s;VCF:-My.vcf.gz;VCF:-multi_chr.vcf.gz;" |\
sed "s;OUT:-MY_PREFIX;OUT:-VICTOR;" > slurm.annotate

chmod +x slurm.annotate

## To execute slurm.annotate, do one of the following:

# 1) submit slurm.annotate to a job scheduler:
# victor.sbatch --array=1 slurm.annotate

# 2) run slurm.annotate directly:
./slurm.annotate 1>slurm.annotate.run_1.stdout 2>slurm.annotate.run_1.stderr
