# cox regression, stratified regression and then meta analysis
# p-value is obtained from a likelihood ratio test and then combined by Fisher's method
# eg: --my-R=etc/myR.coxph_meta.R --R-par="SURV_FILE:/path/to/follow_up_data_and_covariates;SURV_EVENT:Death;SURV_TIME:FU,Death;SURV_COV:+AgeDx+Tstage"
#     If you don't want covariates, you still need to set "SURV_COV:".
# VERSION: 2022-12-21. Ignore invariable covariates for each stratum.

############################
# 0. load packages
############################
options(warn=-1)
suppressWarnings(library(survival))
suppressMessages(library(meta)) # for metagen
suppressMessages(library(metap)) # for sumz


############################
# 1. SET VARIABLES
############################
STRATUM_VAR <- STRATUM_VAR_HEADER # "victorPop"
ONE_SIDED <- one_sided

############################
# 2. customize
############################
EVENT_VAR <- "SURV_EVENT"
y <- read.table("SURV_FILE", header=TRUE)
COV_STR <- "SURV_COV" # "+Sex+AgeDx+AdvStage+Site+Histology"

############################
# 3. FUNCTIONS:
############################
# Identifies which populations to include in the analysis
which_pops <- function(mypop){
  df <- get(paste0("Subset", mypop))
  # include in analysis if (1) the number of events is >= MIN_NUM_EVENTS and
  # (2) the gene column has at least 2 different values and teh >0 values are present at least MIN_CARR_EA_STRA
  if((length(df[[EVENT_VAR]][which((df[[EVENT_VAR]]>0) & (!is.na(df[[EVENT_VAR]])))]) >= MIN_NUM_EVENTS) & (length(unique(df$gene[which(!is.na(df$gene))])) > 1) & (length(df$gene[which((df$gene>0) & (!is.na(df$gene)))]) >= MIN_CARR_EA_STRA)){
    return(TRUE)
  } else {
    return(FALSE)
  }
  rm(myDf, df)
}
 
# extracts beta or standard error from each subsetted data-frame
cox.func <- function(mypop, mystat){
  lmod <- get(paste0("cox.", mypop))
  if (mystat == "beta"){
    return(lmod$coefficients["gene",1])
  }
  if (mystat == "se"){
    return(lmod$coefficients["gene",3])
  }
}
 
# calculates weight for each subsetted dataset for meta-analysis, or extracts p-value from each subsetted dataset
meta.stats <- function(mypop, mystat){
  ## get weight: sqrt of Effective sample size
  if (mystat == "weight"){
    df <- get(paste0("Subset", mypop))
    Neff <- length(df[[EVENT_VAR]][which(df[[EVENT_VAR]] == 1)])
    weight <- sqrt(Neff)
    return(weight)
  } 
  if (mystat == "pval"){
    # get p-value
    lmod <- get(paste0("cox.", mypop))
    return(lmod$lik_rat_pv)
  }
}
 

############################
# 4. Read Data:
############################
 
# MERGE DATA
merged <- merge(x, y, by="SeqID")
final_data <- na.omit(merged)
rm(x,y,merged)
 
if (!is.null(STRATUM_VAR)){
  # get list of unique populations
  my_values <- unique(final_data[[STRATUM_VAR]])
  
  # Subset by STRATUM_VAR_HEADER
  for (i in my_values){
    # subset
    assign(paste0("Subset", i), final_data[which(final_data[[STRATUM_VAR]]==i),]) 
  }
} else {
  my_values <- "All"
  SubsetAll <- final_data
}
 
 
# Determine which stratifications to run regression on (avoid doing regression on anything with an outcome or gene column that only has one unique value).  
test.SV <- my_values[which(unlist(lapply(my_values, function(f) {which_pops(f)})))]
if (length(test.SV) > 0){
  for (i in test.SV){
    THIS_STR <- COV_STR
    covars <- unlist(strsplit(THIS_STR, "\\+"))
    covars <- covars[covars != ""]
    for (j in covars){
      if (length(unique(get(paste0("Subset", i))[[j]]))==1){
        THIS_STR <- gsub(paste0("\\+", j), "", THIS_STR)
      }
    }
    f <- paste0("Surv(SURV_TIME) ~ gene + COVARIATE_NAMES", THIS_STR)
    g <- paste0("Surv(SURV_TIME) ~ COVARIATE_NAMES", THIS_STR)
    print(paste0(i, ": ", f))
    
    full_model<-summary(coxph(as.formula(f), data=get(paste0("Subset", i)), weights=IndWt))
    redu_model<-summary(coxph(as.formula(g), data=get(paste0("Subset", i)), weights=IndWt))
    lik_rat_pv<-pchisq(abs(full_model$loglik[2]-redu_model$loglik[2])*2,1,lower.tail=FALSE)
    result <- c(full_model, lik_rat_pv = list(lik_rat_pv))
    assign(paste0("cox.", i), result)
  }
  
  # Get beta and stderr for each stratification
  beta <- unlist(lapply(test.SV, function (f) {cox.func(f, "beta")}))
  stderr <- unlist(lapply(test.SV, function (f) {cox.func(f, "se")}))
  pvals <- unlist(lapply(test.SV, function (f) {meta.stats(f, "pval")}))
  zscores <- ifelse(beta > 0, qnorm(pvals/2), qnorm(1-(pvals/2)))
  pvals.corrected <- ifelse(beta > 0, pvals/2, (1-(pvals/2)))

  df.stats <- as.data.frame(cbind(test.SV, beta, stderr))
  df.stats$beta <- as.numeric(df.stats$beta)
  df.stats$stderr <- as.numeric(df.stats$stderr)
  if (ONE_SIDED) {
    df.stats$pval <- pvals.corrected
  } else {
    df.stats$pval <- pvals
  }

  # Meta-analysis by sumz
  weights <- unlist(lapply(test.SV, function (f) {meta.stats(f, "weight")}))
  if (length(test.SV) > 1){
    overall <- sumz(p=pvals.corrected, weights=weights)
  } else {
    p<-pvals.corrected[1]
    z<-zscores[1]
    overall <- c(z=list(z), p=list(p))
  }
  if (ONE_SIDED) {
    sumz.pv <- overall$p
  } else {
    sumz.pv <- ifelse(overall$p < 0.5, overall$p*2, (1-overall$p)*2 )
  }  

  # Meta-analysis by metagen
  meta.log <- metagen(TE=df.stats$beta, seTE=df.stats$stderr)

  # Output:
  df.stats$hr <- round(exp(df.stats$beta),2)
  df.stats$se <- abs(df.stats$beta/zscores)
  df.stats$L95 <- round(exp(df.stats$beta - (1.96 * df.stats$se)),2)
  df.stats$U95 <- round(exp(df.stats$beta + (1.96 * df.stats$se)),2)
  SE<-abs(meta.log$TE.fixed/overall$z)
  HR<-exp(meta.log$TE.fixed)
  L95<-exp(meta.log$TE.fixed - (1.96 * SE))
  U95<-exp(meta.log$TE.fixed + (1.96 * SE))
  mystr <- paste0("All:HR=", round(HR,2), "[", round(L95,2), ",", round(U95,2), "]|pv=", signif(sumz.pv,2))
  for (i in my_values[order(my_values)]){
    if (!(i %in% test.SV)){
      mystr <- paste0(mystr, "/", i, ":NA")
    } else {
      index <- which(df.stats$test.SV == i)
      df.stats[index, 2]
      mystr <- paste0(mystr, "/", i, ":HR=", round(df.stats$hr[index],2), "[", round(df.stats$L95[index],2), ",", round(df.stats$U95[index],2), "]|pv=", signif(df.stats$pval[index],2))
    }
  }
  cat("MyR_beta ", meta.log$TE.fixed, "\n", sep = "")
  cat("MyR_pval ", sumz.pv, "\n", sep = "")
  cat("MyR_info ", mystr, "\n", sep = "")
}
